@@ -1,15 +1,14 @@
 from typing import Any, Final, List, Optional, cast
 from eth_typing import Address
+from eth_utils import to_checksum_address
 
 from web3 import Web3
 from web3.eth import TxReceipt
-from web3.contract import ContractFunctions
-from thirdweb.abi.i_signature_mint import ISignatureMint
-from thirdweb.abi.i_thirdweb_platform_fee import IThirdwebPlatformFee
-from thirdweb.abi.i_thirdweb_primary_sale import IThirdwebPrimarySale
+from web3.contract import ContractFunctions, ContractFunction
 from thirdweb.abi.i_token_erc1155 import ITokenERC1155
 from thirdweb.abi.i_token_erc20 import ITokenERC20
 from thirdweb.abi.i_token_erc721 import ITokenERC721
+from thirdweb.common.error import NoSignerException
 from thirdweb.common.feature_detection import matches_interface
 from thirdweb.constants.role import ALL_ROLES
 from thirdweb.core.classes.base_contract import BaseContract
@@ -26,7 +25,13 @@ from thirdweb.core.classes.ipfs_storage import IpfsStorage
 from zero_ex.contract_wrappers.tx_params import TxParams
 from thirdweb.types.contract import ContractType
 from eth_account.account import LocalAccount
-from thirdweb.abi import ThirdwebContract, AccessControlEnumerable, IThirdwebRoyalty
+from thirdweb.abi import (
+    ThirdwebContract,
+    IPermissionsEnumerable,
+    IRoyalty,
+    IPrimarySale,
+    IPlatformFee,
+)
 
 from thirdweb.types.sdk import SDKOptions
 from thirdweb.types.settings.metadata import (
@@ -36,9 +41,38 @@ from thirdweb.types.settings.metadata import (
 
 
 class CustomContract(BaseContract[ThirdwebContract]):
+    """
+    An SDK interface for a custom contract instance.
+
+    ```python
+    from thirdweb import ThirdwebSDK
+
+    # You can customize this to a supported network or your own RPC URL
+    network = "mumbai"
+
+    # Now we can create a new instance of the SDK
+    sdk = ThirdwebSDK(network)
+
+    # If you want to send transactions, you can instantiate the SDK with a private key instead:
+    #   sdk = ThirdwebSDK.from_private_key(PRIVATE_KEY, network)
+
+    contract = sdk.get_contract("{{contract_address}}")
+
+    # If your contract follows the ERC721 standard, contract.nft will be present
+    nfts = contract.nft.get_all()
+
+    # You can call any contract read function as follows
+    balance = contract.functions.balance().call()
+
+    # And you can also use write functions like this
+    address = "0x..."
+    uri = "ipfs://..."
+    contract.send_transaction("mint", [address, uri])
+    ```
+    """
+
     contract_type: Final[ContractType] = ContractType.CUSTOM
 
-    metadata: ContractMetadata[ThirdwebContract, ContractMetadataSchema]
     functions: ContractFunctions
 
     def __init__(
@@ -58,9 +92,6 @@ class CustomContract(BaseContract[ThirdwebContract]):
         self.functions = contract.functions
 
         self._storage = storage
-        self.metadata = ContractMetadata(
-            contract_wrapper, storage, CustomContractMetadata
-        )
 
         self.roles = self._detect_roles()
         self.royalties = self._detect_royalties()
@@ -71,41 +102,70 @@ class CustomContract(BaseContract[ThirdwebContract]):
         self.nft = self._detect_erc_721()
         self.edition = self._detect_erc_1155()
 
-        # self.signature_mint = self.detect_signature_mint()
+    def call(self, fn: str, *args) -> Any:
+        func = cast(ContractFunction, getattr(self.functions, fn, None))
+        if func is None:
+            raise Exception(
+                f"Function {fn} not found on contract {self.get_address()}. "
+                + "Check your dashboard for the list of available functions."
+            )
 
-    def send_transaction(self, fn: str, args: List[Any]) -> TxReceipt:
-        provider = self._contract_wrapper.get_provider()
-        signer = self._contract_wrapper.get_signer()
+        # We need this to set params properly on func + throws good errors
+        func.args = args
+        func.kwargs = {}
+        func._set_function_info()
 
-        nonce = provider.eth.get_transaction_count(signer.address)  # type: ignore
-        tx = getattr(self.functions, fn)(*args).buildTransaction(
-            TxParams(gas_price=provider.eth.gas_price).as_dict()
-        )
-        tx["nonce"] = nonce
+        if len(func.abi["inputs"]) != len(args):
+            signature = (
+                "("
+                + ", ".join(
+                    [(i["name"] + ": " + i["type"]) for i in func.abi["inputs"]]
+                )
+                + ")"
+            )
+            raise Exception(
+                f"Function {fn} expects {len(func.arguments)} arguments, "
+                f"but {len(args)} were provided.\nExpected function signature: {signature}"
+            )
 
-        signed_tx = signer.sign_transaction(tx)  # type: ignore
-        tx_hash = provider.eth.send_raw_transaction(signed_tx.rawTransaction)
-        return provider.eth.wait_for_transaction_receipt(tx_hash)
+        if func.abi["stateMutability"] == "view":
+            return func(*args).call()
+        else:
+            provider = self._contract_wrapper.get_provider()
+            signer = self._contract_wrapper.get_signer()
+
+            if signer is None:
+                raise NoSignerException
+
+            nonce = provider.eth.get_transaction_count(signer.address)  # type: ignore
+
+            tx = func(*args).buildTransaction(
+                TxParams(gas_price=provider.eth.gas_price).as_dict()
+            )
+            tx["nonce"] = nonce
+
+            signed_tx = signer.sign_transaction(tx)  # type: ignore
+            tx_hash = provider.eth.send_raw_transaction(signed_tx.rawTransaction)
+
+            return provider.eth.wait_for_transaction_receipt(tx_hash)
 
     """
     INTERNAL FUNCTIONS
     """
 
     def _detect_roles(self):
-        interface_to_match = self._get_interface_functions(
-            AccessControlEnumerable.abi()
-        )
+        interface_to_match = self._get_interface_functions(IPermissionsEnumerable.abi())
 
         if matches_interface(self.functions, interface_to_match):
-            contract_wrapper = self._get_contract_wrapper(AccessControlEnumerable)
+            contract_wrapper = self._get_contract_wrapper(IPermissionsEnumerable)
             return ContractRoles(contract_wrapper, ALL_ROLES)
         return None
 
     def _detect_royalties(self):
-        interface_to_match = self._get_interface_functions(IThirdwebRoyalty.abi())
+        interface_to_match = self._get_interface_functions(IRoyalty.abi())
 
         if matches_interface(self.functions, interface_to_match):
-            contract_wrapper = self._get_contract_wrapper(IThirdwebRoyalty)
+            contract_wrapper = self._get_contract_wrapper(IRoyalty)
             metadata = ContractMetadata(
                 contract_wrapper,
                 self._storage,
@@ -115,18 +175,18 @@ class CustomContract(BaseContract[ThirdwebContract]):
         return None
 
     def _detect_primary_sales(self):
-        interface_to_match = self._get_interface_functions(IThirdwebPrimarySale.abi())
+        interface_to_match = self._get_interface_functions(IPrimarySale.abi())
 
         if matches_interface(self.functions, interface_to_match):
-            contract_wrapper = self._get_contract_wrapper(IThirdwebPrimarySale)
+            contract_wrapper = self._get_contract_wrapper(IPrimarySale)
             return ContractPrimarySale(contract_wrapper)
         return None
 
     def _detect_platform_fee(self):
-        interface_to_match = self._get_interface_functions(IThirdwebPlatformFee.abi())
+        interface_to_match = self._get_interface_functions(IPlatformFee.abi())
 
         if matches_interface(self.functions, interface_to_match):
-            contract_wrapper = self._get_contract_wrapper(IThirdwebPlatformFee)
+            contract_wrapper = self._get_contract_wrapper(IPlatformFee)
             return ContractPlatformFee(contract_wrapper)
         return None
 
@@ -154,14 +214,6 @@ class CustomContract(BaseContract[ThirdwebContract]):
             return ERC1155(contract_wrapper, self._storage)
         return None
 
-    def _detect_signature_mint(self):
-        interface_to_match = self._get_interface_functions(ISignatureMint.abi())
-
-        if matches_interface(self.functions, interface_to_match):
-            contract_wrapper = self._get_contract_wrapper(ISignatureMint)
-            return ERC1155(contract_wrapper, self._storage)
-        return None
-
     def _get_interface_functions(self, abi: str) -> ContractFunctions:
         return (
             self._contract_wrapper.get_provider()
