@@ -33,15 +33,16 @@ class EditionDrop(ERC1155[DropERC1155]):
 
     ```python
     from thirdweb import ThirdwebSDK
-    from eth_account import Account
 
     # You can customize this to a supported network or your own RPC URL
     network = "mumbai"
 
-    # This will create a random account to use for signing transactions
-    signer = Account.create()
+    # Now we can create a new instance of the SDK
+    sdk = ThirdwebSDK(network)
+
+    # If you want to send transactions, you can instantiate the SDK with a private key instead:
+    #   sdk = ThirdwebSDK.from_private_key(PRIVATE_KEY, network)
 
-    sdk = ThirdwebSDK(network, signer)
     contract = sdk.get_edition_drop("{{contract_address}}")
     ```
     """
@@ -189,6 +190,7 @@ class EditionDrop(ERC1155[DropERC1155]):
             "claim",
             [
                 destination_address,
+                token_id,
                 quantity,
                 claim_verification.currency_address,
                 claim_verification.price,
