@@ -1,16 +1,22 @@
+from eth_account import Account
 from thirdweb.abi.thirdweb_contract import ThirdwebContract
-from thirdweb.common.feature_detection import fetch_contract_metadata
+from thirdweb.common.feature_detection import (
+    fetch_contract_metadata_from_address,
+)
+from thirdweb.constants.urls import get_provider_for_network
 from thirdweb.contracts import Marketplace
 from thirdweb.contracts.custom import CustomContract
 from thirdweb.contracts.edition_drop import EditionDrop
 from thirdweb.contracts.nft_drop import NFTDrop
+from thirdweb.contracts.multiwrap import Multiwrap
+from thirdweb.core.auth.wallet_authenticator import WalletAuthenticator
 from thirdweb.core.classes.contract_deployer import ContractDeployer
 from thirdweb.core.classes.ipfs_storage import IpfsStorage
 from thirdweb.core.classes.provider_handler import ProviderHandler
 from thirdweb.contracts import Token, Edition, NFTCollection
 
 from eth_account.account import LocalAccount
-from typing import Dict, Optional, Type, Union, cast
+from typing import Any, Dict, Optional, Type, Union, cast
 from web3 import Web3
 
 from thirdweb.types.sdk import SDKOptions
@@ -22,15 +28,34 @@ class ThirdwebSDK(ProviderHandler):
     """
 
     __contract_cache: Dict[
-        str, Union[NFTCollection, Edition, Token, Marketplace, NFTDrop, EditionDrop]
+        str,
+        Union[
+            NFTCollection,
+            Edition,
+            Token,
+            Marketplace,
+            NFTDrop,
+            EditionDrop,
+            Multiwrap,
+        ],
     ] = {}
     storage: IpfsStorage
-
+    auth: WalletAuthenticator
     deployer: ContractDeployer
 
+    @staticmethod
+    def from_private_key(
+        private_key: str,
+        network: str,
+        options: SDKOptions = SDKOptions(),
+    ) -> "ThirdwebSDK":
+        signer = Account.from_key(private_key)
+        sdk = ThirdwebSDK(network, signer, options)
+        return sdk
+
     def __init__(
         self,
-        provider: Web3,
+        network: str,
         signer: Optional[LocalAccount] = None,
         options: SDKOptions = SDKOptions(),
         storage: IpfsStorage = IpfsStorage(),
@@ -44,9 +69,12 @@ class ThirdwebSDK(ProviderHandler):
         :param storage: optional IPFS storage instance to use for storing data
         """
 
+        provider = get_provider_for_network(network)
         super().__init__(provider, signer, options)
-        self.storage = storage
+
+        self.auth = WalletAuthenticator(provider, signer, options)
         self.deployer = ContractDeployer(provider, signer, options, storage)
+        self.storage = storage
 
     def get_nft_collection(self, address: str) -> NFTCollection:
         """
@@ -108,28 +136,57 @@ class ThirdwebSDK(ProviderHandler):
 
         return cast(EditionDrop, self._get_contract(address, EditionDrop))
 
-    def get_custom_contract(self, address: str, abi: str = ""):
+    def get_multiwrap(self, address: str) -> Multiwrap:
         """
-        Get an SDK interface for any custom contract! If you deployed the contract with
-        the thirdweb CLI, you won't need to specify an ABI. Alternatively, you can
-        alternatively specify an ABI if you're contract doesn't have one uploaded.
+        Returns a multiwrap contract SDK instance
 
-        :param address: address of the contract
-        :param abi: optional ABI to use for the contract
-        :returns: Custom contract SDK instance
+        :param address: address of the multiwrap contract
+        :returns: multiwrap contract SDK instance
         """
 
-        if not abi:
-            try:
-                contract = ThirdwebContract(self.get_provider(), address)
-                metadata_uri = contract.get_publish_metadata_uri.call()
-                abi = fetch_contract_metadata(metadata_uri, self.storage)
-            except Exception as e:
-                raise Exception("Error fetching ABI for this contract: " + str(e))
+        return cast(Multiwrap, self._get_contract(address, Multiwrap))
 
-        return CustomContract(
-            self.get_provider(), address, abi, self.storage, self.get_signer()
+    def get_contract(self, address: str) -> CustomContract:
+        """
+        Returns a custom contract SDK instance
+
+        :param address: address of the custom contract
+        :returns: custom contract SDK instance
+        """
+
+        if address in self.__contract_cache:
+            return cast(CustomContract, self.__contract_cache[address])
+
+        try:
+            provider = self.get_provider()
+            abi = fetch_contract_metadata_from_address(address, provider, self.storage)
+            return self.get_contract_from_abi(address, abi)
+        except:
+            raise Exception(f"Error fetching ABI for this contract\n{address}")
+
+    def get_contract_from_abi(self, address: str, abi: str) -> CustomContract:
+        """
+        Returns a custom contract SDK instance given the contract ABI
+
+        :param address: address of the custom contract
+        :param abi: abi of the custom contract
+        :returns: custom contract SDK instance
+        """
+
+        if address in self.__contract_cache:
+            return cast(CustomContract, self.__contract_cache[address])
+
+        provider = self.get_provider()
+        contract = CustomContract(
+            provider,
+            address,
+            abi,
+            self.storage,
+            self.get_signer(),
+            self.get_options(),
         )
+        self.__contract_cache[address] = cast(Any, contract)
+        return contract
 
     def update_provider(self, provider: Web3):
         """
@@ -138,7 +195,9 @@ class ThirdwebSDK(ProviderHandler):
         :param provider: web3 provider instance to use for getting on-chain data
         """
 
-        super().update_provider(provider)
+        super()._update_provider(provider)
+        self.auth._update_provider(provider)
+        self.deployer._update_provider(provider)
 
         for contract in self.__contract_cache.values():
             contract.on_provider_updated(provider)
@@ -150,7 +209,9 @@ class ThirdwebSDK(ProviderHandler):
         :param signer: signer to use for sending transactions
         """
 
-        super().update_signer(signer)
+        super()._update_signer(signer)
+        self.auth._update_signer(signer)
+        self.deployer._update_signer(signer)
 
         for contract in self.__contract_cache.values():
             contract.on_signer_updated(signer)
@@ -165,8 +226,11 @@ class ThirdwebSDK(ProviderHandler):
             Type[Marketplace],
             Type[NFTDrop],
             Type[EditionDrop],
+            Type[Multiwrap],
         ],
-    ) -> Union[NFTCollection, Edition, Token, Marketplace, NFTDrop, EditionDrop]:
+    ) -> Union[
+        NFTCollection, Edition, Token, Marketplace, NFTDrop, EditionDrop, Multiwrap
+    ]:
         if address in self.__contract_cache:
             return self.__contract_cache[address]
 
